# -*- coding: utf8 -*-

"""
The function in this Python module determines the current memory usage of the
current process by reading the VmSize value from /proc/$pid/status. It's based
on the following entry in the Python cookbook:
http://code.activestate.com/recipes/286222/
"""

import platform

if platform.system()=='Linux':
    from ._get_memory_usage_linux import get_memory_usage, get_real_memory_usage
else:
    from ._get_memory_usage_other import get_memory_usage, get_real_memory_usage

# vim: ts=2 sw=2 et



# Tests

from tests.base import BaseTestZSTD,tDATA,log,zstd,platform,raise_skip
from.get_memory_usage import get_real_memory_usage
from time import time
import os

class TestZstdSpeed(BaseTestZSTD):

    def test_00_system_info(self):
        log.info("Bundled libzstd uses assembler? : %r" % zstd.ZSTD_with_asm())
        log.info("Bundled libzstd uses threads? :%r" % zstd.ZSTD_with_threads())

    def test_compression_speed3(self):
        wait = 10
        if "ZSTD_FULLTIME_TESTS" in os.environ:
            wait = 30
        log.info("\nWait %d seconds..." % wait)
        sum = 0
        l=len(tDATA)
        tbegin = time()
        beginMemoryUsage=get_real_memory_usage()
        log.info("begin Compression memory usage = %6.2f kb" % (1.0*beginMemoryUsage/1024,))
        while time()-tbegin<wait:
            cdata = zstd.compress(tDATA,3,0)
            sum+=l

        endMemoryUsage=get_real_memory_usage()
        log.info("end Compression memory usage = %6.2f kb" % (1.0*endMemoryUsage/1024,))
        log.info("Compression speed 3 average = %6.2f Mb/sec" % (1.0*sum/1024/1024/wait,))
        log.info("diff Compression memory usage = %6.2f kb" % (1.0*(endMemoryUsage-beginMemoryUsage)/1024,))

    def test_compression_speed19(self):
        wait = 10
        if "ZSTD_FULLTIME_TESTS" in os.environ:
            wait = 30
        log.info("\nWait %d seconds..." % wait)
        sum = 0
        l=len(tDATA)
        tbegin = time()
        beginMemoryUsage=get_real_memory_usage()
        log.info("begin Compression memory usage = %6.2f kb" % (1.0*beginMemoryUsage/1024,))
        while time()-tbegin<wait:
            cdata = zstd.compress(tDATA,19,0)
            sum+=l

        endMemoryUsage=get_real_memory_usage()
        log.info("end Compression memory usage = %6.2f kb" % (1.0*endMemoryUsage/1024,))
        log.info("Compression speed 19 average = %6.2f Mb/sec" % (1.0*sum/1024/1024/wait,))
        log.info("diff Compression memory usage = %6.2f kb" % (1.0*(endMemoryUsage-beginMemoryUsage)/1024,))

    def test_compression_speed_minus1(self):
        wait = 10
        if "ZSTD_FULLTIME_TESTS" in os.environ:
            wait = 30
        log.info("\nWait %d seconds..." % wait)
        sum = 0
        l=len(tDATA)
        tbegin = time()
        beginMemoryUsage=get_real_memory_usage()
        log.info("begin Compression memory usage = %6.2f kb" % (1.0*beginMemoryUsage/1024,))
        while time()-tbegin<wait:
            cdata = zstd.compress(tDATA,-1,0)
            sum+=l

        endMemoryUsage=get_real_memory_usage()
        log.info("end Compression memory usage = %6.2f kb" % (1.0*endMemoryUsage/1024,))
        log.info("Compression speed -1 average = %6.2f Mb/sec" % (1.0*sum/1024/1024/wait,))
        log.info("diff Compression memory usage = %6.2f kb" % (1.0*(endMemoryUsage-beginMemoryUsage)/1024,))

    def test_compression2_speed3_init_context_once(self):
        wait = 10
        if "ZSTD_FULLTIME_TESTS" in os.environ:
            wait = 30
        log.info("\nWait %d seconds..." % wait)
        sum = 0
        l=len(tDATA)
        tbegin = time()
        beginMemoryUsage=get_real_memory_usage()
        log.info("begin Compression2 memory usage = %6.2f kb" % (1.0*beginMemoryUsage/1024,))
        while time()-tbegin<wait:
            cdata = zstd.compress2(tDATA,3,0)
            sum+=l

        endMemoryUsage=get_real_memory_usage()
        log.info("end Compression2 memory usage = %6.2f kb" % (1.0*endMemoryUsage/1024,))
        log.info("Compression2 speed 3 average = %6.2f Mb/sec" % (1.0*sum/1024/1024/wait,))
        log.info("diff Compression2 memory usage = %6.2f kb" % (1.0*(endMemoryUsage-beginMemoryUsage)/1024,))

    def test_compression2_speed19_init_context_once(self):
        wait = 10
        if "ZSTD_FULLTIME_TESTS" in os.environ:
            wait = 30
        log.info("\nWait %d seconds..." % wait)
        sum = 0
        l=len(tDATA)
        tbegin = time()
        beginMemoryUsage=get_real_memory_usage()
        log.info("begin Compression2 memory usage = %6.2f kb" % (1.0*beginMemoryUsage/1024,))
        while time()-tbegin<wait:
            cdata = zstd.compress2(tDATA,19,0)
            sum+=l

        endMemoryUsage=get_real_memory_usage()
        log.info("end Compression2 memory usage = %6.2f kb" % (1.0*endMemoryUsage/1024,))
        log.info("Compression2 speed 19 average = %6.2f Mb/sec" % (1.0*sum/1024/1024/wait,))
        log.info("diff Compression2 memory usage = %6.2f kb" % (1.0*(endMemoryUsage-beginMemoryUsage)/1024,))

    def test_compression2_speed_minus1_init_context_once(self):
        wait = 10
        if "ZSTD_FULLTIME_TESTS" in os.environ:
            wait = 30
        log.info("\nWait %d seconds..." % wait)
        sum = 0
        l=len(tDATA)
        tbegin = time()
        beginMemoryUsage=get_real_memory_usage()
        log.info("begin Compression2 memory usage = %6.2f kb" % (1.0*beginMemoryUsage/1024,))
        while time()-tbegin<wait:
            cdata = zstd.compress2(tDATA,-1,0)
            sum+=l

        endMemoryUsage=get_real_memory_usage()
        log.info("end Compression2 memory usage = %6.2f kb" % (1.0*endMemoryUsage/1024,))
        log.info("Compression2 speed -1 average = %6.2f Mb/sec" % (1.0*sum/1024/1024/wait,))
        log.info("diff Compression2 memory usage = %6.2f kb" % (1.0*(endMemoryUsage-beginMemoryUsage)/1024,))

    def test_compression_speed_no_cpu_cores_cache(self):
        wait = 10
        if "ZSTD_FULLTIME_TESTS" in os.environ:
            wait = 30
        log.info("\nWait %d seconds..." % wait)
        sum = 0
        l=len(tDATA)
        tbegin = time()
        beginMemoryUsage=get_real_memory_usage()
        log.info("begin Compression memory usage = %6.2f kb" % (1.0*beginMemoryUsage/1024,))
        zstd.ZSTD_stopCpuCoresCache()
        while time()-tbegin<wait:
            cdata = zstd.compress(tDATA,3,0)
            sum+=l

        endMemoryUsage=get_real_memory_usage()
        log.info("end Compression memory usage = %6.2f kb" % (1.0*endMemoryUsage/1024,))
        log.info("Compression speed average = %6.2f Mb/sec" % (1.0*sum/1024/1024/wait,))
        log.info("diff Compression memory usage = %6.2f kb" % (1.0*(endMemoryUsage-beginMemoryUsage)/1024,))

    def test_compression_one_thread_speed(self):
        wait = 10
        if "ZSTD_FULLTIME_TESTS" in os.environ:
            wait = 30
        log.info("\nWait %d seconds..." % wait)
        sum = 0
        l=len(tDATA)
        tbegin = time()
        beginMemoryUsage=get_real_memory_usage()
        log.info("begin Compression with one thread memory usage = %6.2f kb" % (1.0*beginMemoryUsage/1024,))
        while time()-tbegin<wait:
            cdata = zstd.compress(tDATA*10,3,1)
            sum+=l

        endMemoryUsage=get_real_memory_usage()
        log.info("end Compression with one thread memory usage = %6.2f kb" % (1.0*endMemoryUsage/1024,))
        log.info("Compression speed with one thread average = %6.2f Mb/sec" % (1.0*sum/1024/1024/wait,))
        log.info("diff Compression with one thread memory usage = %6.2f kb" % (1.0*(endMemoryUsage-beginMemoryUsage)/1024,))

    def test_decompression_block_speed(self):
        wait = 10
        if "ZSTD_FULLTIME_TESTS" in os.environ:
            wait = 30
        log.info("\nWait %d seconds..." % wait)
        sum = 0
        cdata = zstd.compress(tDATA)
        l=len(cdata*10)
        tbegin = time()
        while time()-tbegin<wait:
            data = zstd.decompress(cdata*10)
            sum+=l

        log.info("Decompression of block data speed average = %6.2f Mb/sec" % (1.0*sum/1024/1024/wait,))

    def test_decompression_stream_speed(self):
        wait = 10
        if "ZSTD_FULLTIME_TESTS" in os.environ:
            wait = 30
        log.info("\nWait %d seconds..." % wait)
        sum = 0
#        cdata = zstd.compress(tDATA)
#        cdata = b'\x28\xb5\x2f\xfd\x00\x58\x11\x00\x00\x7b\x7d'
#        log.info('debug cwd: %s' % os.getcwd())
        curdir = os.path.dirname(os.path.abspath(__file__))
#        log.info('curdir: %s' % curdir)
#        if platform.system()=='Windows':
#            raise_skip("Windows can't find tests data")
        f = open(curdir+"/test_data/facebook.ico.zst","rb")
        cdata = f.read()
        f.close()
        l=len(cdata*10)
        tbegin = time()
        while time()-tbegin<wait:
            data = zstd.decompress(cdata*10)
            sum+=l

        log.info("Decompression of streamed data speed average = %6.2f Mb/sec" % (1.0*sum/1024/1024/wait,))


    def test_check_speed(self):
        wait = 10
        if "ZSTD_FULLTIME_TESTS" in os.environ:
            wait = 30
        log.info("\nWait %d seconds..." % wait)
        sum = 0
        cdata = zstd.compress(tDATA)
        l=len(cdata*10)
        tbegin = time()
        beginMemoryUsage=get_real_memory_usage()
        log.info("begin Check memory usage = %6.2f kb" % (1.0*beginMemoryUsage/1024,))
        while time()-tbegin<wait:
            data = zstd.check(cdata*10)
            sum+=l

        endMemoryUsage=get_real_memory_usage()
        log.info("end Check memory usage = %6.2f kb" % (1.0*endMemoryUsage/1024,))
        log.info("Check speed average = %6.2f Mb/sec" % (1.0*sum/1024/1024/wait,))
        log.info("diff Check memory usage = %6.2f kb" % (1.0*(endMemoryUsage-beginMemoryUsage)/1024,))

    def test_cpu_cores_cache_default_speed(self):
        wait = 10
        if "ZSTD_FULLTIME_TESTS" in os.environ:
            wait = 70
        log.info("\nWait %d seconds..." % wait)
        ops = 0
        tbegin = time()
        beginMemoryUsage=get_real_memory_usage()
        log.info("begin Check memory usage= %6.2f kb" % (1.0*beginMemoryUsage/1024,))
        while time()-tbegin<wait:
            cores = zstd.ZSTD_threads_count()
            ops+=1

        endMemoryUsage=get_real_memory_usage()
        log.info("end Check memory usage = %6.2f kb" % (1.0*endMemoryUsage/1024,))
        log.info("Check cache use speed(default) average = %6.2f Ops/sec" % (1.0*ops/wait,))
        log.info("diff Check memory usage = %6.2f kb" % (1.0*(endMemoryUsage-beginMemoryUsage)/1024,))

    def test_cpu_cores_cache_none_speed(self):
        wait = 10
        if "ZSTD_FULLTIME_TESTS" in os.environ:
            wait = 70
        log.info("\nWait %d seconds..." % wait)
        ops = 0
        tbegin = time()
        beginMemoryUsage=get_real_memory_usage()
        zstd.ZSTD_stopCpuCoresCache()
        log.info("begin Check memory usage= %6.2f kb" % (1.0*beginMemoryUsage/1024,))
        while time()-tbegin<wait:
            cores = zstd.ZSTD_threads_count()
            ops+=1

        endMemoryUsage=get_real_memory_usage()
        log.info("end Check memory usage = %6.2f kb" % (1.0*endMemoryUsage/1024,))
        log.info("Check cache use speed(0) average = %6.2f Ops/sec" % (1.0*ops/wait,))
        log.info("diff Check memory usage = %6.2f kb" % (1.0*(endMemoryUsage-beginMemoryUsage)/1024,))
    
    def test_cpu_cores_cache_60_speed(self):
        wait = 10
        if "ZSTD_FULLTIME_TESTS" in os.environ:
            wait = 70
        log.info("\nWait %d seconds..." % wait)
        ops = 0
        tbegin = time()
        beginMemoryUsage=get_real_memory_usage()
        log.info("begin Check memory usage= %6.2f kb" % (1.0*beginMemoryUsage/1024,))
        while time()-tbegin<wait:
            cores = zstd.ZSTD_threads_count()
            ops+=1

        endMemoryUsage=get_real_memory_usage()
        log.info("end Check memory usage = %6.2f kb" % (1.0*endMemoryUsage/1024,))
        log.info("Check cache use speed(60) average = %6.2f Ops/sec" % (1.0*ops/wait,))
        log.info("diff Check memory usage = %6.2f kb" % (1.0*(endMemoryUsage-beginMemoryUsage)/1024,))
        
    def test_cpu_cores_cache_01_speed(self):
        wait = 10
        if "ZSTD_FULLTIME_TESTS" in os.environ:
            wait = 70
        log.info("\nWait %d seconds..." % wait)
        ops = 0
        tbegin = time()
        beginMemoryUsage=get_real_memory_usage()
        zstd.ZSTD_setCpuCoresCacheTTL(1)
        log.info("begin Check memory usage= %6.2f kb" % (1.0*beginMemoryUsage/1024,))
        while time()-tbegin<wait:
            cores = zstd.ZSTD_threads_count()
            ops+=1

        endMemoryUsage=get_real_memory_usage()
        log.info("end Check memory usage = %6.2f kb" % (1.0*endMemoryUsage/1024,))
        log.info("Check cache use speed(1) average = %6.2f Ops/sec" % (1.0*ops/wait,))
        log.info("diff Check memory usage = %6.2f kb" % (1.0*(endMemoryUsage-beginMemoryUsage)/1024,))

    def test_cpu_cores_cache_05_speed(self):
        wait = 10
        if "ZSTD_FULLTIME_TESTS" in os.environ:
            wait = 70
        log.info("\nWait %d seconds..." % wait)
        ops = 0
        tbegin = time()
        beginMemoryUsage=get_real_memory_usage()
        zstd.ZSTD_setCpuCoresCacheTTL(5)
        log.info("begin Check memory usage= %6.2f kb" % (1.0*beginMemoryUsage/1024,))
        while time()-tbegin<wait:
            cores = zstd.ZSTD_threads_count()
            ops+=1

        endMemoryUsage=get_real_memory_usage()
        log.info("end Check memory usage = %6.2f kb" % (1.0*endMemoryUsage/1024,))
        log.info("Check cache use speed(5) average = %6.2f Ops/sec" % (1.0*ops/wait,))
        log.info("diff Check memory usage = %6.2f kb" % (1.0*(endMemoryUsage-beginMemoryUsage)/1024,))

    def test_cpu_cores_cache_10_speed(self):
        wait = 10
        if "ZSTD_FULLTIME_TESTS" in os.environ:
            wait = 70
        log.info("\nWait %d seconds..." % wait)
        ops = 0
        tbegin = time()
        beginMemoryUsage=get_real_memory_usage()
        zstd.ZSTD_setCpuCoresCacheTTL(10)
        log.info("begin Check memory usage= %6.2f kb" % (1.0*beginMemoryUsage/1024,))
        while time()-tbegin<wait:
            cores = zstd.ZSTD_threads_count()
            ops+=1

        endMemoryUsage=get_real_memory_usage()
        log.info("end Check memory usage = %6.2f kb" % (1.0*endMemoryUsage/1024,))
        log.info("Check cache use speed(10) average = %6.2f Ops/sec" % (1.0*ops/wait,))
        log.info("diff Check memory usage = %6.2f kb" % (1.0*(endMemoryUsage-beginMemoryUsage)/1024,))
    
if __name__ == '__main__':
    unittest.main()
  



# -*- coding: utf8 -*-

"""
The function in this Python module determines the current memory usage of the
current process by reading the VmSize value from /proc/$pid/status. It's based
on the following entry in the Python cookbook:
http://code.activestate.com/recipes/286222/
"""

# simulation for maxos and windows not having /proc
def get_memory_usage():
    return 0

def get_real_memory_usage():
    return 0
# vim: ts=2 sw=2 et



# Tests

import sys, os
from tests.base import BaseTestZSTD, zstd, tDATA, log, raise_skip, platform

class TestZstdDecompress(BaseTestZSTD):

    def test_decompression_null(self):
        if sys.hexversion < 0x03000000:
            DATA = ''
        else:
            DATA = b''
        self.assertRaises(zstd.Error, zstd.uncompress, zstd.compress(DATA)+b' ')

    def test_decompression_streamed(self):
        log.info('debug cwd: %s' % os.getcwd())
        curdir = os.path.dirname(os.path.abspath(__file__))
        log.info('curdir: %s' % curdir)
        if platform.system()=='Windows':
            raise_skip("Windiows can't find tests data")
        f = open(curdir+"/test_data/facebook.ico.zst","rb")
        DATA = f.read()
        f.close()
        log.info('data check, should be 2: %s' % zstd.check(DATA))
        self.assertEqual(2, zstd.check(DATA))
        zstd.uncompress(DATA)
        #self.assertRaises(zstd.Error, zstd.uncompress, DATA)

    def test_decompression_rusted(self):
        #if sys.hexversion < 0x03000000:
        #raise_skip("need python version >= 3")
        if sys.hexversion < 0x03000000:
            data = '{}'
        else:
            data = b'{}'
        cdata = b'\x28\xb5\x2f\xfd\x00\x58\x11\x00\x00\x7b\x7d'
        log.info('data check, should be 2: %s' % zstd.check(cdata))
        self.assertEqual(2, zstd.check(cdata))
        log.info("data must be '{}': %r" % zstd.uncompress(cdata))
        self.assertEqual(data, zstd.uncompress(cdata))
        
    def test_check_compressed(self):
        cdata = zstd.compress(tDATA)
        check = zstd.check(cdata)
        log.info("zstd compressed data check, must be (1):%r" % check)
        self.assertEqual(1, check)

    def test_check_not_compressed(self):
        check = zstd.check(tDATA)
        log.info("zstd not compressed data check, must be (0):%r" % check)
        self.assertEqual(0, check)
        
    def test_check_uncompressed(self):
        cdata = b''
        log.info("zstd uncompressed data check, must be (0):%r" % zstd.check(cdata))
        self.assertEqual(0, zstd.check(cdata))
        



# -*- coding: utf-8 -*-

import os
import sys
import logging

import unittest

# Dirty hack, on 2.6 tests are ok always
# UnitTest2 not clearly works with setuptools
if sys.version_info < (2, 7):
    def raise_skip(msg):
        return None
else:
    def raise_skip(msg):
        raise unittest.SkipTest(msg)

import zstd
import platform

logging.basicConfig(level=logging.INFO)
log = logging.getLogger('ZSTD')
log.info("Python version: %s" % sys.version)
log.info("Machine:%s; processor:%s; system:%r; release:%r" % ( platform.machine(), platform.processor(), platform.system(), platform.release()))
log.info("Module exported interfac e:%r"% dir(zstd))
log.info("libzstd linked external: %r"% zstd.ZSTD_external())
log.info("libzstd built with legacy formats support: %r"% zstd.ZSTD_legacy_support())
log.info("zstd max number of threads: %r"% zstd.ZSTD_max_threads_count())
log.info("zstd found CPU cores : %r"% zstd.ZSTD_threads_count())
log.info("zstd default compression level: %r"% zstd.ZSTD_default_compression_level())
log.info("zstd max compression level: %r"% zstd.ZSTD_max_compression_level())
log.info("zstd min compression level: %r"% zstd.ZSTD_min_compression_level())
log.info("pyzstd module version: %r"% zstd.version())
log.info("libzstd version: %r"% zstd.ZSTD_version())
log.info("Compiled with libzstd version: %r"% zstd.ZSTD_version_compiled())
log.info("Loaded with libzstd version: %r"% zstd.ZSTD_version_loaded())


# Classic lorem ipsum
# + За словесными горами
bDATA = 'Lorem ipsum dolor sit amet, consectetuer adipiscing elit, sed diam nonummy nibh euismod tincidunt ut laoreet dolore magna aliquam erat volutpat.'+\
    ' Ut wisi enim ad minim veniam, quis nostrud exerci tation ullamcorper suscipit lobortis nisl ut aliquip ex ea commodo consequat.'+\
    ' Duis autem vel eum iriure dolor in hendrerit in vulputate velit esse molestie consequat, vel illum dolore eu feugiat nulla facilisis at vero eros et accumsan'+\
    ' et iusto odio dignissim qui blandit praesent luptatum zzril delenit augue duis dolore te feugait nulla facilisi.'
uDATA =\
    ' И немного юникода.'+\
    ' Далеко-далеко за словесными горами в стране гласных и согласных живут рыбные тексты.'+\
    ' Вдали от всех живут они в буквенных домах на берегу Семантика большого языкового океана.'+\
    ' Маленький ручеек Даль журчит по всей стране и обеспечивает ее всеми необходимыми правилами.'+\
    ' Эта парадигматическая страна, в которой жаренные члены предложения залетают прямо в рот.'+\
    ' Даже всемогущая пунктуация не имеет власти над рыбными текстами, ведущими безорфографичный образ жизни.'
tDATA = bDATA + uDATA
if sys.hexversion >= 0x03000000:
    tDATA = tDATA.encode()

class BaseTestZSTD(unittest.TestCase):

    VERSION = "1.5.7"
    VERSION_INT = 10507
    # minimal external version, must be >=1.4.0
    VERSION_INT_MIN = 1 * 100*100 + 4* 1*100 + 0
    PKG_VERSION = "1.5.7.3"

    def helper_version(self):
        #if zstd.ZSTD_external():
        #    return raise_skip("PyZstd was build with external version of ZSTD library, so module is like (%s). It can be any version. Almost." % zstd.version())
        self.assertEqual(self.PKG_VERSION, zstd.version())

    def helper_zstd_version(self):
        if zstd.ZSTD_external():
            return raise_skip("PyZstd was build with external version of ZSTD library (%s). It can be any version. Almost." % zstd.ZSTD_version())
        self.assertEqual(self.VERSION, zstd.ZSTD_version())

    def helper_zstd_version_number_min(self):
        self.assertFalse(self.VERSION_INT_MIN > zstd.ZSTD_version_number(), msg="PyZstd %s require external library version >= 1.0.0!" % zstd.version())

    def helper_zstd_version_number(self):
        if zstd.ZSTD_external():
            return raise_skip("PyZstd was build with external version of ZSTD library (%s). It can be any version. Almost." % zstd.ZSTD_version())
        self.assertEqual(self.VERSION_INT, zstd.ZSTD_version_number())

    def helper_compression_random(self):
        DATA = os.urandom(128 * 1024)  # Read 128kb
        self.assertEqual(DATA, zstd.loads(zstd.dumps(DATA)))

    def helper_compression_default_level(self):
        CDATA = zstd.compress(tDATA)
        self.assertEqual(tDATA, zstd.decompress(CDATA))

    def helper_compression_default_level_number(self):
        self.assertEqual(3, zstd.ZSTD_default_compression_level())

    def helper_compression_default_level_zero(self):
        CDATA = zstd.compress(tDATA)
        self.assertEqual(CDATA, zstd.compress(tDATA, 0))

    def helper_compression_default_level_default(self):
        CDATA = zstd.compress(tDATA)
        self.assertEqual(CDATA, zstd.compress(tDATA, 3))

    def helper_compression_negative_level(self):
        if zstd.ZSTD_version_number() < 10304:
            return raise_skip("PyZstd was build with old version of ZSTD library (%s) without support of negative compression levels." % zstd.ZSTD_version())

        CDATA = zstd.compress(tDATA, -1)
        self.assertEqual(tDATA, zstd.decompress(CDATA))

    def helper_compression_negative_level_notdefault(self):
        if zstd.ZSTD_version_number() < 10304:
            return raise_skip("PyZstd was build with old version of ZSTD library (%s) without support of negative compression levels." % zstd.ZSTD_version())

        CDATA = zstd.compress(tDATA, -1)
        self.assertNotEqual(CDATA, zstd.compress(tDATA, 0))

    #def helper_compression_wrong_level(self):
    #    self.assertRaises(zstd.Error, zstd.compress, tDATA, 100)

    def helper_compression_multi_thread_one(self):
        CDATA = zstd.compress(tDATA, 6, 1)
        self.assertEqual(tDATA, zstd.decompress(CDATA))

    def helper_compression_multi_thread_many(self):
        CDATA = zstd.compress(tDATA, 6, 16)
        self.assertEqual(tDATA, zstd.decompress(CDATA))

    def helper_compression_level1(self):
        if sys.hexversion < 0x03000000:
            DATA = 'This is must be very very long string to be compressed by zstd. AAAAAAAAAAAAARGGHHH!!! Just hope its enough length. И немного юникода.'
        else:
            DATA = b'This is must be very very long string to be compressed by zstd. AAAAAAAAAAAAARGGHHH!!! Just hope its enough length.' + 'И немного юникода.'.encode()
        self.assertEqual(DATA, zstd.decompress(zstd.compress(DATA, 1)))

    def helper_compression_level6(self):
        if sys.hexversion < 0x03000000:
            DATA = "This is must be very very long string to be compressed by zstd. AAAAAAAAAAAAARGGHHH!!! Just hope its enough length. И немного юникода."
        else:
            DATA = b"This is must be very very long string to be compressed by zstd. AAAAAAAAAAAAARGGHHH!!! Just hope its enough length." + " И немного юникода.".encode()
        self.assertEqual(DATA, zstd.decompress(zstd.compress(DATA, 6)))

    def helper_compression_level20(self):
        if sys.hexversion < 0x03000000:
            DATA = "This is must be very very long string to be compressed by zstd. AAAAAAAAAAAAARGGHHH!!! Just hope its enough length. И немного юникода."
        else:
            DATA = b"This is must be very very long string to be compressed by zstd. AAAAAAAAAAAAARGGHHH!!! Just hope its enough length." + " И немного юникода.".encode()
        self.assertEqual(DATA, zstd.decompress(zstd.compress(DATA, 20)))

    def helper_compression_empty_string(self):
        # https://github.com/sergey-dryabzhinsky/python-zstd/issues/80
        # An empty string should be able to be compressed and decompressed
        # All levels 1-20 are tested, just to be safe.
        data = b""

        for level in range(0, 20):
            self.assertEqual(data, zstd.decompress(zstd.compress(data, level + 1)))

    def helper_compression_multiple_blocks(self):
        # https://github.com/sergey-dryabzhinsky/python-zstd/issues/94
        # An conctenaed blocks should be able to be decompressed
        if sys.hexversion < 0x03000000:
            import codecs
            data = codecs.decode("28b52ffd200631000068656c6c6f0a28b52ffd2006310000776f726c640a", 'hex_codec')
            odata = "hello\nworld\n"
        else:
            data = bytes.fromhex("28b52ffd200631000068656c6c6f0a28b52ffd2006310000776f726c640a")
            odata = b"hello\nworld\n"

        self.assertEqual(odata, zstd.decompress(data))



# -*- coding: utf8 -*-

"""
The function in this Python module determines the current memory usage of the
current process by reading the VmSize value from /proc/$pid/status. It's based
on the following entry in the Python cookbook:
http://code.activestate.com/recipes/286222/
"""

import os

_units = { 'KB': 1024, 'MB': 1024**2, 'GB': 1024**3 }
_handle = open('/proc/%d/status' % os.getpid())

def get_memory_usage():
    global _proc_status, _units, _handle
    try:
        for line in _handle:
            if line.startswith('VmSize:'):
                label, count, unit = line.split()
        return int(count) * _units[unit.upper()]
    except:
        return 0
    finally:
        _handle.seek(0)

def get_real_memory_usage():
    global _proc_status, _units, _handle
    try:
        for line in _handle:
            if line.startswith('VmRSS:'):
                label, count, unit = line.split()
        return int(count) * _units[unit.upper()]
    except:
        return 0
    finally:
        _handle.seek(0)
# vim: ts=2 sw=2 et



#!/usr/bin/env python

import os
import sys

#debug
# print("\ncmdline: %r" % (sys.argv,))

import subprocess
import platform 

import setuptools
from setuptools import setup, find_packages, Extension
from setuptools.command.build_ext import build_ext

# bundled ZSTD version
VERSION = (1, 5, 7,)
VERSION_STR = ".".join([str(x) for x in VERSION])


# Package version, even external 
PKG_VERSION = VERSION
# Minor revision 
PKG_VERSION += (3,)
PKG_VERSION_STR = ".".join([str(x) for x in PKG_VERSION])
vpath="./version"
if not os.path.exists(vpath):
	vdn=os.path.dirname( os.path.abspath(sys.argv[0]))
	vpath=os.path.join(vdn,vpath)
if os.path.exists(vpath):
	print("use existing %s file" % vpath)
	f = open(vpath,"r" )
PKG_VERSION_STR = f.read().strip()
f.close()

def which(bin_exe):
    """
    Simulate shutil.which for python 2.7,
    which available only since python 3.3.
    Search executable in common paths.

    @return: False or full path to executable 
    @rtype: str|bool
    """
    # support Linux / POSIX here? Or maybe windows?
    paths = ["/usr/local/bin", "/usr/local/sbin", "/bin", "/sbin", "/usr/bin", "/usr/sbin"]
    if "PATH" in os.environ:
        paths = os.environ["PATH"].split(":")
    for p in paths:
        full_path = os.path.join(p, bin_exe)
        if os.path.isfile(full_path) and os.access(full_path, os.X_OK):
            return full_path
    return False
###
# Ugly hacks, I know
#

SUP_LEGACY="ZSTD_LEGACY" in os.environ
if "--legacy" in sys.argv:
    # Support legacy output format functions
    if SUP_LEGACY:
        if os.environ[ZSTD_LEGACY]=='0':
            SUP_LEGACY=False
        else:
            SUP_LEGACY=True
    sys.argv.remove("--legacy")

SUP_WARNINGS="ZSTD_WARNINGS" in os.environ
if SUP_WARNINGS:
    if os.environ["ZSTD_WARNINGS"]=='0':
        SUP_WARNINGS=False
    else:
        SUP_WARNINGS=True
if "--all-warnings" in sys.argv:
    SUP_WARNINGS=True
    sys.argv.remove("--all-warnings")

SUP_WERROR="ZSTD_WERRORS" in os.environ
if SUP_WERROR:
    if os.environ["ZSTD_WERRORS"]=='0':
        SUP_WERROR=False
    else:
        SUP_WERROR=True
if "--all-warnings-errors" in sys.argv:
    # Support legacy output format functions
    SUP_WERROR=True
    sys.argv.remove("--all-warnings-errors")
    
SUP_ASM="ZSTD_ASM" in os.environ
if "ZSTD_ASM" not in os.environ:
    SUP_ASM = False
#a asm off by default
if "--libzstd-use-asm" in sys.argv:
    # Support assembler builtin optimization in lizstd
    SUP_ASM=True
    sys.argv.remove("--libzstd-use-asm")
DISABLE_ASM=0
if not SUP_ASM:
     DISABLE_ASM=1

SUP_THREADS="ZSTD_THREADS" in os.environ 
if "ZSTD_THREADS" not in os.environ:
    SUP_THREADS = True
else:
    if os.environ['ZSTD_THREADS']=='0':
        SUP_THREADS = False
# threads on by default
if "--libzstd-no-threads" in sys.argv:
    # Disable support multithreading in lizstd
    SUP_THREADS=False
    sys.argv.remove("--libzstd-no-threads")
ENABLE_THREADS=0
if SUP_THREADS:
     ENABLE_THREADS=1

SUP_ASM_BMI2="ZSTD_ASM_BMI2" in os.environ
if "--libzstd-use-asm-bmi2" in sys.argv:
    # Support assembler builtin optimization in lizstd for new AMD CPU
    SUP_ASM_BMI2=True 
    sys.argv.remove("--libzstd-use-asm-bmi2")
ENABLE_ASM_BMI2=0
if SUP_ASM_BMI2:
     ENABLE_ASM_BMI2=1

SUP_TRACE="ZSTD_TRACE" in os.environ
if "--debug-trace" in sys.argv:
    # Support tracing for debug
    SUP_TRACE=True
    sys.argv.remove("--debug-trace")

BUILD_SPEED0="ZSTD_SPEED0" in os.environ
if "--speed0" in sys.argv:
    # speed or size choose only one
    if BUILD_SPEED0:
        if os.environ["ZSTD_SPEED0"]=='0':
            BUILD_SPEED0=False
    BUILD_SPEED0=True
    BUILD_SMALL=False
    sys.argv.remove("--speed0")
else:
    BUILD_SPEED0=False

BUILD_SPEED1="ZSTD_SPEED1" in os.environ
if "--speed1" in sys.argv:
    if BUILD_SPEED1:
        if os.environ["ZSTD_SPEED1"]=='0':
            BUILD_SPEED1=False
    # speed or size choose only one
    BUILD_SPEED1=True
    BUILD_SMALL=False
    sys.argv.remove("--speed1")
else:
    BUILD_SPEED1=False

BUILD_SPEED2="ZSTD_SPEED2" in os.environ
if "--speed2" in sys.argv:
    if BUILD_SPEED2:
        if os.environ["ZSTD_SPEED2"]=='0':
            BUILD_SPEED2=False
    # speed or size choose only one
    BUILD_SPEED2=True
    BUILD_SMALL=False
    sys.argv.remove("--speed2")
else:
    BUILD_SPEED2=False

BUILD_SPEED3="ZSTD_SPEED3" in os.environ
if "--speed3" in sys.argv:
    # speed or size choose only one
    BUILD_SPEED3=True
    BUILD_SMALL=False
    sys.argv.remove("--speed3")
else:
    BUILD_SPEED3=True
if BUILD_SPEED3:
    if os.environ.get("ZSTD_SPEED3")=='0':
        BUILD_SPEED3=False

BUILD_SPEEDMAX="ZSTD_SPEEDMAX" in os.environ
if "--speed-max" in sys.argv:
    # speed or size choose only one
    BUILD_SPEED3=True
    BUILD_SPEEDMAX=True 
    BUILD_SMALL=False
    sys.argv.remove("--speed-max")

BUILD_SMALL="ZSTD_SMALL" in os.environ
if "--small" in sys.argv:
    if BUILD_SMALL:
        if os.environ["ZSTD_SMALL"]=='0':
            BUILD_SMALL=False
    BUILD_SMALL=True
    BUILD_SPEED3=False
    sys.argv.remove("--small")
else:
    BUILD_SMALL=False

SUP_EXTERNAL="ZSTD_EXTERNAL" in os.environ
ext_libraries=[]
if "--external" in sys.argv:
    # You want use external Zstd library?
    SUP_EXTERNAL=True
    sys.argv.remove("--external")

SUP_DEBUG="ZSTD_DEBUG" in os.environ
if SUP_DEBUG:
    if os.environ["ZSTD_DEBUG"]=='0':
        SUP_DEBUG=False
ext_libraries=[]
if "--debug" in sys.argv:
    SUP_DEBUG=True
    sys.argv.remove("--debug")

SUP_DEBUG_NOTICE="ZSTD_DEBUG_NOTICE" in os.environ
if SUP_DEBUG_NOTICE:
    SUP_DEBUG=True
    if os.environ["ZSTD_DEBUG_NOTICE"]=='0':
        SUP_DEBUG_NOTICE=False
ext_libraries=[]
if "--debug-notice" in sys.argv:
    SUP_DEBUG_NOTICE=True
    SUP_DEBUG=True
    sys.argv.remove("--debug-notice")

SUP_DEBUG_INFO="ZSTD_DEBUG_INFO" in os.environ
if SUP_DEBUG_INFO:
    SUP_DEBUG=True
    if os.environ["ZSTD_DEBUG_INFO"]=='0':
        SUP_DEBUG_INFO=False
ext_libraries=[]
if "--debug-info" in sys.argv:
    SUP_DEBUG_INFO=True
    SUP_DEBUG=True
    sys.argv.remove("--debug-info")

SUP_DEBUG_ERROR="ZSTD_DEBUG_ERROR" in os.environ
if SUP_DEBUG_ERROR:
    SUP_DEBUG=True
    if os.environ["ZSTD_DEBUG_ERROR"]=='0':
        SUP_DEBUG_ERROR=False
ext_libraries=[]
if "--debug-error" in sys.argv:
    SUP_DEBUG_ERROR=True
    SUP_DEBUG=True
    sys.argv.remove("--debug-error")

#Some python builds need to disable LTO by force
BUILD_NO_LTO="ZSTD_BUILD_NO_LTO" in os.environ
if BUILD_NO_LTO:
    if os.environ["ZSTD_BUILD_NO_LTO"]=='0':
        BUILD_NO_LTO=False
if "--force-no-lto" in sys.argv:
    BUILD_NO_LTO=True
    sys.argv.remove("--force-no-lto")
else:
    BUILD_NO_LTO=True

BUILD_STRIP="ZSTD_BUILD_STRIP" in os.environ
if BUILD_STRIP:
    if os.environ["ZSTD_BUILD_STRIP"]=='0':
        BUILD_STRIP=False
if "--force-strip" in sys.argv:
    BUILD_STRIP=True
    sys.argv.remove("--force-strip")
else:
    BUILD_STRIP=True
    
pkgconf = which("pkg-config")
if "--libzstd-bundled" in sys.argv:
    # Do you want use external Zstd library?
    SUP_EXTERNAL=False
    sys.argv.remove("--libzstd-bundled")
    pkgconf = False

#if SUP_EXTERNAL:
if platform.system() == "Linux" and "build_ext" in sys.argv or "build" in sys.argv or "bdist_wheel" in sys.argv or "test" in sys.argv:
    # You should add external library by option: --libraries zstd
    # And probably include paths by option: --include-dirs /usr/include/zstd
    # And probably library paths by option: --library-dirs /usr/lib/i386-linux-gnu
    # We need pkg-config here!
    if pkgconf is not False:
        #debug 
        #print("pkg-config exists")
        cmd = [pkgconf, "libzstd", "--modversion"]
        if sys.hexversion >= 0x03000000:
            VERSION_STR=b''
        else:
            VERSION_STR=""
        if sys.hexversion >= 0x02070000:
            try:
                VERSION_STR = subprocess.check_output(cmd).strip()
            except Exception as e:
                print("Error: %r" % e) 
                pass
        else:
            # Pure Python 2.6
            VERSION_STR = subprocess.Popen(cmd, stdout=subprocess.PIPE).communicate()[0].strip()
        if sys.hexversion >= 0x03000000:
            # It's bytes in PY3
            VERSION_STR = VERSION_STR.decode()
        #debug
        if SUP_DEBUG:
            print("\nFound libzstd version %r" % VERSION_STR)
        if VERSION_STR and SUP_EXTERNAL:
            if VERSION_STR>="1.4.0":
                SUP_EXTERNAL=True
                if "--libraries" not in sys.argv:
                    # Add something default
                    ext_libraries=["zstd"]
            else:
                raise RuntimeError("Need zstd library verion >= 1.4.0")
            #VERSION = tuple(int(v) for v in VERSION_STR.split("."))
    else:
        if SUP_EXTERNAL:
            # Require pkg config
            raise RuntimeError("Need pkg-config to find system libzstd.")
        print("\n Need pkg-config to find system libzstd. Or we try bundled one.")


# default standard optimization
COPT = {
    'msvc': ['/O2', ],
    'mingw32': ['-O2',],
    'unix': ['-O2',],
    'clang': ['-O2',],
    'gcc': ['-O2',],
}
if BUILD_SMALL:
   COPT = {
       'msvc': ['/O1', ],
       'mingw32': ['-Os',],
       'unix': ['-Os',],
       'clang': ['-Os',],
       'gcc': ['-Os',],
   }

# not small, but speed?
if BUILD_SPEED0:
    COPT = {
        'msvc': ['/O0', ],
        'mingw32': ['-O0',],
        'unix': ['-O0',],
        'clang': ['-O0',],
        'gcc': ['-O0',],
    }
if BUILD_SPEED1:
    COPT = {
        'msvc': ['/O1', ],
        'mingw32': ['-O1',],
        'unix': ['-O1',],
        'clang': ['-O1',],
        'gcc': ['-O1',],
    }
if BUILD_SPEED2:
    COPT = {
        'msvc': ['/O2', ],
        'mingw32': ['-O3',],
        'unix': ['-O3',],
        'clang': ['-O3',],
        'gcc': ['-O3',],
    }
if BUILD_SPEED3:
    COPT = {
        'msvc': ['/Ox', ],
        'mingw32': ['-O3',],
        'unix': ['-O3',],
        'clang': ['-O3',],
        'gcc': ['-O3',],
    }
###

for comp in COPT:
    if comp == 'msvc':
        COPT[comp].extend([ '/DMOD_VERSION=%s' % PKG_VERSION_STR, '/DDYNAMIC_BMI2=%d' % ENABLE_ASM_BMI2, '/DZSTD_DISABLE_ASM=%d' % DISABLE_ASM ]),
    else:
        COPT[comp].extend([ '-DMOD_VERSION=%s' % PKG_VERSION_STR, '-DDYNAMIC_BMI2=%d' % ENABLE_ASM_BMI2, '-DZSTD_DISABLE_ASM=%d' % DISABLE_ASM ]),

if not SUP_EXTERNAL:
    for comp in COPT:
        if comp == 'msvc':
            COPT[comp].extend([ '/DZSTD_MULTITHREAD=%d' % ENABLE_THREADS,
                '/Izstd\\lib', '/Izstd\\lib\\common', '/Izstd\\lib\\compress', '/Izstd\\lib\\decompress',
            ])
        else:
            COPT[comp].extend([ '-DZSTD_MULTITHREAD=%d' % ENABLE_THREADS,
                '-Izstd/lib', '-Izstd/lib/common', '-Izstd/lib/compress', '-Izstd/lib/decompress',
            ])
else:
    for comp in COPT:
        if comp == 'msvc':
            COPT[comp].extend([ '/DLIBZSTD_EXTERNAL=1'
            ])
        else:
            COPT[comp].extend([ '-DLIBZSTD_EXTERNAL=1',
            ])

if SUP_DEBUG:
    for comp in COPT:
        if comp == 'msvc':
            COPT[comp].extend([ '/DZSTD_DEBUG=1','-g',
            ])
        else:
            COPT[comp].extend([ '-DZSTD_DEBUG=1','-g',
            ])

if BUILD_NO_LTO:
    for comp in COPT:
        if comp == 'msvc':
            pass
        else:
            COPT[comp].extend([ '-fno-lto',
            ])

if SUP_DEBUG_NOTICE:
    for comp in COPT:
        if comp == 'msvc':
            COPT[comp].extend([ '/DZSTD_DEBUG_NOTICE=1',
            ])
        else:
            COPT[comp].extend([ '-DZSTD_DEBUG_NOTICE=1',
            ])

if SUP_DEBUG_INFO:
    for comp in COPT:
        if comp == 'msvc':
            COPT[comp].extend([ '/DZSTD_DEBUG_INFO=1',
            ])
        else:
            COPT[comp].extend([ '-DZSTD_DEBUG_INFO=1',
            ])

if SUP_DEBUG_ERROR:
    for comp in COPT:
        if comp == 'msvc':
            COPT[comp].extend([ '/DZSTD_DEBUG_ERROR=1',
            ])
        else:
            COPT[comp].extend([ '-DZSTD_DEBUG_ERROR=1',
            ])

if BUILD_SPEEDMAX:
    for comp in COPT:
        if comp == 'msvc':
            COPT[comp].extend([ ''
            ])
        else:
            COPT[comp].extend([ '-march=native',
            ])

if SUP_LEGACY:
    for comp in COPT:
        if comp == 'msvc':
            COPT[comp].extend(['/Izstd\\lib\\legacy', '/DZSTD_LEGACY_SUPPORT=1'])
        else:
            COPT[comp].extend(['-Izstd/lib/legacy', '-DZSTD_LEGACY_SUPPORT=1'])

if SUP_WARNINGS:
    for comp in COPT:
        if comp == 'msvc':
            COPT[comp].extend(['/Wall',])
        else:
            COPT[comp].extend(['-Wall', '-Wextra', '-Wpedantic'])

if SUP_WERROR:
    for comp in COPT:
        if comp == 'msvc':
            COPT[comp].extend(['//WX'])
        else:
            COPT[comp].extend(['-Werror'])

if BUILD_STRIP:
    for comp in COPT:
        if comp == 'msvc':
            pass
        else:
            COPT[comp].extend(['-Wl,-s'])

# Force traceing support or disable
if SUP_TRACE:
    for comp in COPT:
        if comp == 'msvc':
            COPT[comp].extend(['/DZSTD_TRACE=1'])
        else:
            COPT[comp].extend(['-DZSTD_TRACE=1'])
else:
    for comp in COPT:
        if comp == 'msvc':
            COPT[comp].extend(['/DZSTD_TRACE=0'])
        else:
            COPT[comp].extend(['-DZSTD_TRACE=0'])

# protect from python packaged flags that prevent from exporting symbols
for comp in COPT:
    if comp == 'msvc':
        pass
    else:
        COPT[comp].extend(['-std=c99','-fvisibility=default'])

# disable some msvc warnings
for comp in COPT:
    if comp == 'msvc':
        COPT[comp].extend(['/wd4996'])


class ZstdBuildExt( build_ext ):

    def build_extensions(self):
        c = self.compiler.compiler_type
        if c in COPT:
           for e in self.extensions:
               e.extra_compile_args = COPT[c]
        build_ext.build_extensions(self)

zstdFiles = []

if not SUP_EXTERNAL:

    for f in [
            'compress/zstd_compress.c',
            'compress/zstd_compress_literals.c',
            'compress/zstd_compress_sequences.c',
            'compress/zstd_compress_superblock.c',
            'compress/zstd_preSplit.c',
            'compress/zstdmt_compress.c',
            'compress/zstd_fast.c',
            'compress/zstd_double_fast.c',
            'compress/zstd_lazy.c',
            'compress/zstd_opt.c',
            'compress/zstd_ldm.c',
            'compress/fse_compress.c',
            'compress/huf_compress.c',
            'compress/hist.c',

            'common/fse_decompress.c',
            'decompress/zstd_decompress.c',
            'decompress/zstd_decompress_block.c',
            'decompress/zstd_ddict.c',
            'decompress/huf_decompress.c',

            'common/entropy_common.c',
            'common/zstd_common.c',
            'common/xxhash.c',
            'common/error_private.c',
            'common/pool.c',
            'common/threading.c',
        ]:
        zstdFiles.append('zstd/lib/'+f)

    if SUP_LEGACY:
        for f in [
            'legacy/zstd_v01.c', 'legacy/zstd_v02.c', 'legacy/zstd_v03.c', 'legacy/zstd_v04.c', 'legacy/zstd_v05.c', 'legacy/zstd_v06.c', 'legacy/zstd_v07.c'
            ]:
            zstdFiles.append('zstd/lib/'+f)

# files needed always, even for external
zstdFiles.append('src/debug.c')
zstdFiles.append('src/util.c')
zstdFiles.append('src/python-zstd.c')


# Another dirty hack
def my_test_suite():
    import unittest

    os.environ["VERSION"] = VERSION_STR
    os.environ["PKG_VERSION"] = PKG_VERSION_STR

    test_suite = unittest.TestSuite()
    test_suite.addTest(unittest.defaultTestLoader.loadTestsFromName("tests.test_compress"))
    test_suite.addTest(unittest.defaultTestLoader.loadTestsFromName("tests.test_decompress"))
    test_suite.addTest(unittest.defaultTestLoader.loadTestsFromName("tests.test_version"))
    test_suite.addTest(unittest.defaultTestLoader.loadTestsFromName("tests.test_speed"))
    return test_suite

test_func_name = "setup.my_test_suite"
if "test" in sys.argv:
    from setuptools.version import __version__ as version
    print("setuptools version %r" % version)
    if version < '72.0':
        test_func_name = "tests"

f=open('README.rst', 'r')
ld=f.read()
f.close()

if SUP_DEBUG:
	print("debug: ext_libraries:%r" % (ext_libraries))
setup(
    name='zstd',
    version=PKG_VERSION_STR,
    description="ZSTD Bindings for Python",
    long_description=ld,
    author='Sergey Dryabzhinsky, Anton Stuk',
    author_email='sergey.dryabzhinsky@gmail.com',
    maintainer='Sergey Dryabzhinsky',
    maintainer_email='sergey.dryabzhinsky@gmail.com',
    url='https://github.com/sergey-dryabzhinsky/python-zstd',
    keywords=['zstd', 'zstandard', 'compression'],
    license='BSD',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    ext_modules=[
        Extension('zstd', zstdFiles, libraries=ext_libraries)
    ],
    cmdclass = {'build_ext': ZstdBuildExt },
    test_suite=test_func_name,
    classifiers=[
#        'License :: OSI Approved :: BSD License',
        
        'Intended Audience :: Developers',
        'Development Status :: 5 - Production/Stable',
        'Operating System :: POSIX',
        'Programming Language :: C',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Programming Language :: Python :: 3.13',
        'Programming Language :: Python :: 3.14',
        'Programming Language :: Python :: 3.15',
    ]
)



# Tests

from tests.base import BaseTestZSTD

class TestZstdVersion(BaseTestZSTD):

    def test_module_version(self):
        BaseTestZSTD.helper_version(self)

    def test_library_version_number_min(self):
        BaseTestZSTD.helper_zstd_version_number_min(self)

    def test_library_version(self):
        BaseTestZSTD.helper_zstd_version(self)

    def test_library_version_number(self):
        BaseTestZSTD.helper_zstd_version_number(self)

if __name__ == '__main__':
    unittest.main()



# Tests

from tests.base import BaseTestZSTD, zstd, tDATA, log

class TestZstdCompress(BaseTestZSTD):

    def setup(self):
        pass
#        th = zstd.Thread_pool_init()
#        log.info("Stated pool of %d threads" % th)

    def tearDown(self):
        pass
#        th = zstd.Thread_pool_free()
#        log.info("Freed pool of %d threads" % th)

    def test_compression_random(self):
        BaseTestZSTD.helper_compression_random(self)

    def test_compression_default_level(self):
        BaseTestZSTD.helper_compression_default_level(self)

    def test_compression_level_default_is_3(self):
        self.assertEqual(3, zstd.ZSTD_default_compression_level())

    def test_compression_level_max_is_22(self):
        self.assertEqual(22, zstd.ZSTD_max_compression_level())

    def test_compression_level_min_is_m100(self):
        self.assertEqual(-100, zstd.ZSTD_min_compression_level())

    def test_compression_default_level_zero(self):
        BaseTestZSTD.helper_compression_default_level_zero(self)

    def test_compression_default_level_default(self):
        BaseTestZSTD.helper_compression_default_level_default(self)

    def test_compression_negative_level(self):
        BaseTestZSTD.helper_compression_negative_level(self)

    def test_compression_negative_level_notdefault(self):
        BaseTestZSTD.helper_compression_negative_level_notdefault(self)

    def test_compression_wrong_level_strict(self):
        self.assertRaises(zstd.Error, zstd.compress, tDATA, 100, 0, 1)

    def test_compression_equal_high_level(self):
        cdata1 = zstd.compress(tDATA, 22)
        cdata2 = zstd.compress(tDATA, 2200)
        self.assertEqual(cdata1, cdata2)

    def test_compression_MT_and_not(self):
        cdata1 = zstd.compress(tDATA, 14)
        cdata2 = zstd.compress_real_mt(tDATA, 14)
        self.assertEqual(cdata1, cdata2)

    def test_compression_oldMT_and_new2(self):
        cdata1 = zstd.compress(tDATA, 14)
        cdata2 = zstd.compress_real_mt(tDATA, 14)
        self.assertEqual(cdata1, cdata2)

    def test_compression_MT_and_raw(self):
        cdata2 = zstd.compress_real_mt(tDATA, 14)
        data2 = zstd.decompress(cdata2)
        self.assertEqual(tDATA, data2)

    def test_compression_equal_too_low_level(self):
        cdata1 = zstd.compress(tDATA, -100)
        cdata2 = zstd.compress(tDATA, -1000)
        self.assertEqual(cdata1, cdata2)

    def test_compression_equal_too_high_threads(self):
        cdata1 = zstd.compress(tDATA, 22, 256)
        cdata2 = zstd.compress(tDATA, 22, 2560)
        self.assertEqual(cdata1, cdata2)
        
    def test_compression_multi_thread_one(self):
        BaseTestZSTD.helper_compression_multi_thread_one(self)

    def test_compression_multi_thread_many(self):
        BaseTestZSTD.helper_compression_multi_thread_many(self)

    def test_compression_level1(self):
        BaseTestZSTD.helper_compression_level1(self)

    def test_compression_level6(self):
        BaseTestZSTD.helper_compression_level6(self)

    def test_compression_level20(self):
        BaseTestZSTD.helper_compression_level20(self)

    def test_compression_empty_string(self):
        BaseTestZSTD.helper_compression_empty_string(self)

    def test_compression_multiple_blocks(self):
        BaseTestZSTD.helper_compression_multiple_blocks(self)


if __name__ == '__main__':
    unittest.main()
